use serde::{Deserialize, Serialize};

#[repr(u8)]
#[derive(Debug, Clone, Serialize, Deserialize)]
enum MessageType {
    Query,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Command {
    device_protocol_version: u8,
    device_type: u8,
    message_type: MessageType,
    body_type: u8,
}

impl Command {
    pub fn sub_type(device_type: u32) -> Self {
        // Self {}
        todo!()
    }

    pub fn query(device_type: u8, device_protocol_version: u8) -> Self {
        Self {
            device_protocol_version,
            device_type,
            message_type: MessageType::Query,
            body_type: 0x00,
        }
    }
}

impl Command {
    const HEADER_LENGTH: u8 = 10;

    pub fn header(&self) -> Vec<u8> {
        let length = Self::HEADER_LENGTH + self.body().len() as u8;

        vec![
            // flag
            0xAA,
            // length
            length,
            // device type
            self.device_type,
            // frame checksum
            0x00, // self._device_type ^ length,
            // unused
            0x00,
            0x00,
            // frame ID
            0x00,
            // frame protocol version
            0x00,
            // device protocol version
            self.device_protocol_version,
            // frame type
            self.message_type as u8,
        ]
    }

    pub fn body(&self) -> Vec<u8> {
        todo!()
    }
}
